import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

public class BillingApplet extends java.applet.Applet
{
	java.applet.AppletContext appContext;

	private static AppBar pAppBar;
	private AppModule invoicesModule;
	private AppModule paymentsModule;
	private AppModule adjustmentsModule;
	private AppModule reportsModule;
	private AppModule toolsModule;

	public void init()
	{
		//reset all global objects
		AppConfig.reset();

		this.setLayout(null);
		this.setSize(AppConfig.TotalWidth, AppConfig.TotalHeight);
		this.setBackground(Color.lightGray);

		//get Applet Context
		appContext = this.getAppletContext();

		//set reference to this applet
		AppConfig.setBillingApplet(this);

		//Initialize Messaging for this Applet. Must do this before adding
		//any component to Applet.
		XToolkit.initMessaging(this);
		//XToolkit.showWaitMsg("Loading. Please wait..........");

		//Add AppBar to hold Main App Menu and Misc stuff
		pAppBar = new AppBar();
		this.add(pAppBar);
		AppConfig.setAppBar(pAppBar);	//set it for global referencing

		//Add Separator between Client and AppBar
		Canvas tc = new Canvas();
		tc.setBounds(AppConfig.AppBarWidth, 0, AppConfig.SepBarWidth, AppConfig.TotalHeight);
		tc.setBackground(AppConfig.Color_Browser);
		this.add(tc);

		//set customer panel
		AppConfig.setCustomerPanel(new CustomerPanel());
	}

	public void setModule(String mName)
	{
		if(AppConfig.portalURL == null) return;
		if(mName.equals(AppModule.moduleName[AppModule.INVOICES])) loadInvoicesModule();
		else if(mName.equals(AppModule.moduleName[AppModule.PAYMENTS])) loadPaymentsModule();
		else if(mName.equals(AppModule.moduleName[AppModule.ADJUSTMENTS])) loadAdjustmentsModule();
		else if(mName.equals(AppModule.moduleName[AppModule.REPORTS])) loadReportsModule();
		else if(mName.equals(AppModule.moduleName[AppModule.TOOLS])) loadToolsModule();
		else return;
	}

	public void setIspvn(int ispIndex)
	{
		AppConfig.curIspvnIndex = ispIndex;
		AppConfig.getCustomerPanel().reset();
		AppModule activeModule = AppConfig.getActiveModule();
		if(activeModule != null) activeModule.reset();
	}

	private void loadInvoicesModule()
	{
		//Show wait message box
		//XToolkit.showWaitMsg("Loading. Please wait..........");
//System.out.println("Going to sleep......");
//		try { Thread.sleep(5000); } catch(Exception e) { }
//System.out.println("Unhiding it......");
		//XToolkit.hideWaitMsg();

		AppModule cApp = AppConfig.getActiveModule();
		if(invoicesModule == null)
		{
			invoicesModule = ModuleLoader.getInvoicesModule();
			this.add(invoicesModule.getModuleContainer());
		}
		else if(cApp == invoicesModule) return;
		if(cApp != null) cApp.setActive(false);
		invoicesModule.setActive(true);
	}

	private void loadPaymentsModule()
	{
		AppModule cApp = AppConfig.getActiveModule();
		if(paymentsModule == null)
		{
			paymentsModule = ModuleLoader.getPaymentsModule();
			this.add(paymentsModule.getModuleContainer());
		}
		else if(cApp == paymentsModule) return;
		if(cApp != null) cApp.setActive(false);
		paymentsModule.setActive(true);
	}

	private void loadAdjustmentsModule()
	{
		AppModule cApp = AppConfig.getActiveModule();
		if(adjustmentsModule == null)
		{
			adjustmentsModule = ModuleLoader.getAdjustmentsModule();
			this.add(adjustmentsModule.getModuleContainer());
		}
		else if(cApp == adjustmentsModule) return;
		if(cApp != null) cApp.setActive(false);
		adjustmentsModule.setActive(true);
	}

	private void loadReportsModule()
	{
		AppModule cApp = AppConfig.getActiveModule();
		if(reportsModule == null)
		{
			reportsModule = ModuleLoader.getReportsModule();
			this.add(reportsModule.getModuleContainer());
		}
		else if(cApp == reportsModule) return;
		if(cApp != null) cApp.setActive(false);
		reportsModule.setActive(true);
	}

	private void loadToolsModule()
	{
		AppModule cApp = AppConfig.getActiveModule();
		if(toolsModule == null)
		{
			toolsModule = ModuleLoader.getToolsModule();
			this.add(toolsModule.getModuleContainer());
		}
		else if(cApp == toolsModule) return;
		if(cApp != null) cApp.setActive(false);
		toolsModule.setActive(true);
	}

	//Communication with JavaScript
	public void setPortalURL(String u) { AppConfig.portalURL = u; AppConfig.getCommunicator().setURL(u); }

	public void addIspvnName(String iN)
	{
		int tLen = AppConfig.ispvnNameList.length;
		String[] tnList = new String[tLen+1];
		if(tLen > 0) System.arraycopy(AppConfig.ispvnNameList, 0, tnList, 0, tLen);
		tnList[tLen] = iN;
		AppConfig.ispvnNameList = tnList;
	}
	public void addIspvnID(String iD)
	{
		int tLen = AppConfig.ispvnIDList.length;
		String[] tiList = new String[tLen+1];
		if(tLen > 0) System.arraycopy(AppConfig.ispvnIDList, 0, tiList, 0, tLen);
		tiList[tLen] = iD;
		AppConfig.ispvnIDList  = tiList;
	}

	public void refreshIspvnData() { pAppBar.refreshISPList(); }

	public void setOperatorID(String opId) { AppConfig.operatorID = opId; }

	public void setCommApplet(Object co)
	{
		System.out.println("set Applet Called...");
		//java.applet.Applet ca = (java.applet.Applet)co;
		System.out.println(co);
		java.util.Enumeration eApp = appContext.getApplets();
		System.out.println(eApp);
		while(eApp.hasMoreElements())
		{
			Object ta = eApp.nextElement();
			System.out.println(ta);
		}
		//if(ca == null) System.out.println("Null Applet"); else System.out.println("Not Null !");//else ca.TriggerComm();

	}

	public String getAppMessage() { return AppConfig.getHtmlMessage(); }

	public String getHtmlString() { return AppConfig.getHtmlData(); }

	public String testJS()	//should be removed later...
	{
		String msg = "Portal URL: "+AppConfig.portalURL;
		msg = msg.concat("\nIspvnList Length: "+AppConfig.ispvnNameList.length);
		return msg;
	}
}
