import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

public class XGrid extends Panel implements ActionListener, MouseListener, MouseMotionListener
{
	private Font  titleFont    = new Font("SansSerif", Font.BOLD, 13);
	private Font  headerFont   = new Font("SansSerif", Font.PLAIN, 13);
	private Font  dataFont     = new Font("SansSerif", Font.PLAIN, 12);
	private FontMetrics dataFM = Toolkit.getDefaultToolkit().getFontMetrics(dataFont);
	private int dataFontHeight = dataFM.getHeight();

	private Color headerColor    = new Color(168, 168, 168);
	private Color contentColorB  = headerColor;
	private Color contentColorF  = new Color(228, 228, 228);
	private Color dataColor      = Color.black;
	private Color cursorColor    = Color.red;
	private Color selectedColor  = Color.blue;
	private Color inLinkColor    = Color.cyan;

	private int  gHeaderHeight  = 20;
	private Button[] gHeaderBtn;

	private int gridWidth=0, gridHeight=0, gridTop=0;
	private int contentX=0,  contentY=0;
	private int contentWidth=0, contentHeight=0;
	private int borderH=1, borderV=1;

	//Layout specific
	private int   gRowHeight = 20;
	private int[] gColWidth;
	private int[] gColMLeft;
	private int[] gColMRight;
	private int[] gColMTop;
	private int[] gColMBot;
	private int[] gColHAlign;
	private Rectangle[] gColBounds;

	Button btnNextPage, btnPrevPage;

	//***************  New Definitions for XGrid *****************
	public static final int XDATA_ATTRIBUTES   = 101;
	public static final int XDATA_CHILDREN     = 102;
	public static final int XDATA_TEXTCHILDREN = 103;

	public static final int VIEW_MULTIPAGED = 201;
	public static final int VIEW_SCROLLING  = 202;
	public static final int defaultPageSize = 10;

	public static final int PROP_MARGINLEFT  = 401;
	public static final int PROP_MARGINRIGHT = 402;
	public static final int PROP_MARGINTOP   = 403;
	public static final int PROP_MARGINBOT   = 404;
	public static final int PROP_HALIGN      = 405;
	public static final int PROP_VALIGN      = 406;
	public static final int PROP_TEXTCOLOR   = 407;
	public static final int PROP_TEXTFONT    = 408;
	public static final int PROP_DATALINK    = 409;

	private int xDataFormat = XDATA_ATTRIBUTES;
	private int xViewFormat = VIEW_MULTIPAGED;
	private int xPageSize   = defaultPageSize;
	private boolean showPageTabs = false, layoutDone=false;

	private String[][] gridColData;	//Col-Row Data
	private int[][] colDataX;			//Col-Row Data X position
	private int[][] colDataY;			//Col-Row Data Y position
	private boolean[] isColDataLink;	//indicates if the column contains links
	private Color[] gColTextColor;		//Column Text Color
	private Font[]  gColTextFont;		//Column Text Font
	private int gridRowCount, gridColCount, gridPageCount;
	private int topRowIndex=0, selectedRowIndex=-1, curPageIndex, curPageSize;

	//to handle mouse in and out of links columns
	private int mouseInCol=-1, mouseInRow=-1, mouseInColN=-1, mouseInRowN=-1;
	boolean isLinkColEvent=false;
	ActionListener linkListener = null;
	String linkActionCommand="";
	//************************************************************

	public XGrid()
	{
		super();
		gridColCount = 1;
		initXGrid();
	}
	public XGrid(int cols)
	{
		super();
		gridColCount = cols;
		initXGrid();
	}
	public XGrid(String Title)
	{
		this();
		Label tL = new Label(Title);
		tL.setFont(titleFont);
		tL.setBounds(0, 0, 70, 18);
		this.add(tL);
	}

	private void initXGrid()
	{
		this.setLayout(null);
		this.addMouseListener(this);
		this.addMouseMotionListener(this);

		btnNextPage = new Button(">>");
		btnNextPage.setBackground(headerColor);
		btnNextPage.setFont(AppConfig.Font_B11);
		btnNextPage.addActionListener(this);
		this.add(btnNextPage);

		btnPrevPage = new Button("<<");
		btnPrevPage.setBackground(headerColor);
		btnPrevPage.setFont(AppConfig.Font_B11);
		btnPrevPage.addActionListener(this);
		this.add(btnPrevPage);

		//set Layout specific defaults
		gColWidth  = new int[gridColCount];
		gColMLeft  = new int[gridColCount];
		gColMRight = new int[gridColCount];
		gColMTop   = new int[gridColCount];
		gColMBot   = new int[gridColCount];
		gColHAlign = new int[gridColCount];
		gColBounds = new Rectangle[gridColCount];
		isColDataLink = new boolean[gridColCount];
		gColTextColor  = new Color[gridColCount];
		gColTextFont   = new Font[gridColCount];
		for(int i=0; i < gridColCount; i++)
		{	gColWidth[i] = 80;
			gColTextColor[i] = dataColor;
			gColTextFont[i]  = dataFont;
		}
	}

	public void setDataFormat(int format) { xDataFormat = format; }

	public void setViewFormat(int format) { xViewFormat = format; }

	public void setRowHeight(int rh) { gRowHeight = rh; };

	public void setPageSize(int psize) { xPageSize = psize; }

	public void setColumns(int[] cw)
	{
		gColWidth = cw;
		gridColCount = cw.length;
		gColMLeft  = new int[gridColCount];
		gColMRight = new int[gridColCount];
		gColMTop   = new int[gridColCount];
		gColMBot   = new int[gridColCount];
		gColHAlign = new int[gridColCount];
		gColBounds = new Rectangle[gridColCount];
		isColDataLink = new boolean[gridColCount];
		gColTextColor  = new Color[gridColCount];
		gColTextFont   = new Font[gridColCount];
		for(int i=0; i < gridColCount; i++)
		{	gColTextColor[i] = dataColor;
			gColTextFont[i]  = dataFont;
		}
	}

	public void setColProperty(int col, int prop, Object val)
	{
		int i, iVal;

		switch(prop)
		{
		case PROP_MARGINLEFT:
			iVal = ((Integer)val).intValue();
			if(col < 0) for(i=0; i < gridColCount; i++) gColMLeft[i] = iVal;
			else gColMLeft[col] = iVal;
			return;

		case PROP_MARGINRIGHT:
			iVal = ((Integer)val).intValue();
			if(col < 0) for(i=0; i < gridColCount; i++) gColMRight[i] = iVal;
			else gColMRight[col] = iVal;
			return;

		case PROP_MARGINTOP:
			iVal = ((Integer)val).intValue();
			if(col < 0) for(i=0; i < gridColCount; i++) gColMTop[i] = iVal;
			else gColMTop[col] = iVal;
			return;

		case PROP_MARGINBOT:
			iVal = ((Integer)val).intValue();
			if(col < 0) for(i=0; i < gridColCount; i++) gColMBot[i] = iVal;
			else gColMBot[col] = iVal;
			return;

		case PROP_HALIGN:
			iVal = ((Integer)val).intValue();
			if(col < 0) for(i=0; i < gridColCount; i++) gColHAlign[i] = iVal;
			else gColHAlign[col] = iVal;
			return;

		case PROP_TEXTCOLOR:
			Color tc = (Color)val;
			if(col < 0) for(i=0; i < gridColCount; i++) gColTextColor[i] = tc;
			else gColTextColor[col] = tc;
			return;

		case PROP_TEXTFONT:
			Font tf = (Font)val;
			if(col < 0) for(i=0; i < gridColCount; i++) gColTextFont[i] = tf;
			else gColTextFont[col] = tf;
			return;

		case PROP_DATALINK:
			boolean bv = (val == null) ? false : true;
			if(col < 0) for(i=0; i < gridColCount; i++) isColDataLink[i] = bv;
			else isColDataLink[col] = bv;
			return;
		}
	}

	public void setBorderH(int bh) { borderH = bh; }
	public void setBorderV(int bv) { borderV = bv; }

	public void setHeader(String[] hText)
	{
		if(hText == null) { gHeaderBtn = null; return; }

		gHeaderBtn = new Button[hText.length];
		for(int i=0; i < hText.length; i++)
		{	gHeaderBtn[i] = new Button(hText[i]); super.add(gHeaderBtn[i]); }
	}

	public void displayPageTabs(boolean disp) { showPageTabs = disp; }

	//Normally this method should not be called externally
	//This function calculates and sets all layout specific
	//properties of the grid
	public void doLayout()
	{
		gridWidth  = this.getSize().width;
		gridHeight = this.getSize().height;
		if(showPageTabs)
		{	btnNextPage.setBounds(gridWidth-35, 0, 35, 17);
			btnPrevPage.setBounds(gridWidth-73, 0, 35, 17);
			btnNextPage.setVisible(true);
			btnPrevPage.setVisible(true);
			if(gridPageCount > 1) btnNextPage.setEnabled(true);
			else btnNextPage.setEnabled(false);
			btnPrevPage.setEnabled(false);
			gridTop = 18;
		}
		else
		{	btnNextPage.setVisible(false);
			btnPrevPage.setVisible(false);
			gridTop = 0;
		}

		contentX = 1;
		if(gHeaderBtn == null) contentY = gridTop+1;
		else contentY = gridTop + gHeaderHeight + 1;
		contentWidth  = gridWidth - contentX -1;
		contentHeight = gridHeight - contentY -1;

		for(int i=0; i < gridColCount; i++)
			gColBounds[i] = new Rectangle(borderH, borderV, gColWidth[i]-borderH, gRowHeight-borderV);

		if(gHeaderBtn == null) return;
		int sx=2;
		for(int n=0; n < gHeaderBtn.length; n++)
		{	gHeaderBtn[n].setBounds(sx, gridTop+2, gColWidth[n], gHeaderHeight);
			sx += gColWidth[n];
		}

		layoutDone = true;
	}

	public void setContentBackground(Color c) { contentColorB = c; }

	public void setContentForeground(Color c) { contentColorF = c; }

	public void setDataColor(Color c) { dataColor = c; }

	public void setCursorColor(Color c) { cursorColor = c; }

	public void setDataFont(Font f)
	{
		dataFont = f;
		dataFM = Toolkit.getDefaultToolkit().getFontMetrics(f);
		dataFontHeight = dataFM.getHeight();
	}

	public void setLinkListener(ActionListener aL, String cmd)
	{
		linkListener = aL;
		if(cmd != null) linkActionCommand = cmd;
	}

	public void reset()
	{
		gridColData = null;
		colDataX = null;
		colDataY = null;

		gridRowCount = 0;
		gridPageCount= 0;
		topRowIndex  = 0;
		curPageIndex = 0;
		curPageSize  = 0;
		selectedRowIndex = -1;
		mouseInCol = -1;
		mouseInRow = -1;

		this.repaint();
	}

	public void update(Graphics g) { paint(g); }

	public void paint(Graphics g)
	{
		if(isLinkColEvent)	//just redraw mouse columns and return
		{
			if(mouseInCol != -1)
			{	g.setColor(gColTextColor[mouseInCol]);
				g.setFont(gColTextFont[mouseInCol]);
				/*g.setClip(colDataX[mouseInCol][mouseInRow+topRowIndex],
						  colDataY[mouseInCol][mouseInRow+topRowIndex],
						  gColWidth[mouseInCol], gRowHeight);
				*/g.drawString(gridColData[mouseInCol][mouseInRow+topRowIndex],
								 colDataX[mouseInCol][mouseInRow+topRowIndex],
								 colDataY[mouseInCol][mouseInRow+topRowIndex]);
			}
			if(mouseInColN != -1)
			{	g.setColor(inLinkColor);
				g.setFont(gColTextFont[mouseInColN]);
				/*g.setClip(colDataX[mouseInColN][mouseInRowN+topRowIndex],
						  colDataY[mouseInColN][mouseInRowN+topRowIndex],
						  gColWidth[mouseInColN], gRowHeight);
				*/g.drawString(gridColData[mouseInColN][mouseInRowN+topRowIndex],
								 colDataX[mouseInColN][mouseInRowN+topRowIndex],
								 colDataY[mouseInColN][mouseInRowN+topRowIndex]);
				setCursor(AppConfig.Cursor_HAND);
			}
			else setCursor(AppConfig.Cursor_DEFAULT);
			mouseInCol = mouseInColN;  mouseInRow = mouseInRowN;
			isLinkColEvent = false;
			return;
		}

		g.setClip(0, 0, gridWidth, gridHeight);
		//draw 3D box around the grid
		XToolkit.setColorScheme(XToolkit.COLORSCHEME_SHARP3D);
		XToolkit.draw3DBox(g, 0, gridTop, gridWidth, gridHeight-gridTop, false);

		//draw Header
		int hX = 2, hY = gridTop+2, hW = gridWidth-4;
		g.setColor(headerColor);
		g.fillRect(contentX, gridTop+1, contentWidth, gHeaderHeight);
		//fill the content Area
		g.setColor(contentColorF);
		g.fillRect(contentX, contentY, contentWidth, contentHeight);

		//draw Data Page
		int cX = contentX;
		//draw each Column
		for(int i=0; i < gridColCount; i++)
		{
			g.setClip(cX, 0, gColWidth[i], gridHeight);
			//draw vertical border
			if(borderV > 0)
			{	g.setColor(contentColorB);
				g.drawLine(cX, contentY, cX, contentY+curPageSize*gRowHeight-1);
			}
			//draw each row
			for(int j=0; j < curPageSize; j++)
			{	//draw Horizontal border
				if(borderH > 0)
				//if(borderH > 0 && i == 0)
				{	g.setColor(contentColorB);
					g.drawLine(contentX, contentY+j*gRowHeight, contentX+contentWidth-1, contentY+j*gRowHeight);
				}
				g.setColor(gColTextColor[i]);
				g.setFont(gColTextFont[i]);
				g.drawString(gridColData[i][j+topRowIndex],
								 colDataX[i][j+topRowIndex], colDataY[i][j+topRowIndex]);
			}
			cX += gColWidth[i];
		}

		//fill the empty space with contentBackground
		g.setClip(0, 0, gridWidth, gridHeight);
		if(curPageSize < xPageSize)
		{	int pHeight = curPageSize*gRowHeight;
			g.setColor(contentColorB);
			g.fillRect(contentX, contentY+pHeight, contentWidth, contentHeight-pHeight);
		}
	}


	//this method loads data into grid from xml document
	public void loadDocument(Document doc)
	{
		int i, j;

		this.reset();

		Element  dRoot = doc.getDocumentElement();
		NodeList dRows = dRoot.getChildNodes();
		gridRowCount = dRows.getLength();
		gridColData = new String[gridColCount][gridRowCount];
		colDataX = new int[gridColCount][gridRowCount];
		colDataY = new int[gridColCount][gridRowCount];

		//populate the data from document
		if(xDataFormat == XDATA_ATTRIBUTES)
			for(i=0; i < gridRowCount; i++)
			{	Node cRow = dRows.item(i);
				NamedNodeMap attrMap = cRow.getAttributes();
				for(j=0; j < gridColCount; j++)
					gridColData[j][i] = attrMap.item(j).getNodeValue();
			}
		else if(xDataFormat == XDATA_CHILDREN)
			for(i=0; i < gridRowCount; i++)
			{	Node cRow = dRows.item(i);
				NodeList childCols = cRow.getChildNodes();
				for(j=0; j < gridColCount; j++)
					gridColData[j][i] = childCols.item(j).getFirstChild().getNodeValue();
			}
		else for(i=0; i < gridRowCount; i++)
			{	Node cRow = dRows.item(i);
				NodeList childCols = cRow.getChildNodes();
				for(j=0; j < gridColCount; j++)
					gridColData[j][i] = childCols.item(j).getNodeValue();
			}

		updateView();
	}


	void updateView()
	{
		//update page variables
		if(gridRowCount > 0)
		{	gridPageCount = gridRowCount/xPageSize;
			if( (gridRowCount % xPageSize) > 0) gridPageCount++;
		}
		else gridPageCount = 0;
		if(gridPageCount > 1) curPageSize = xPageSize;
		else if(gridPageCount > 0) curPageSize = gridRowCount;
		else curPageSize = 0;

		//set layout parameters
		doLayout();

if(false) {
System.out.println("//***************** GRID Variables ************************");
System.out.println("Header Buttons: "+gHeaderBtn);
//System.out.println("", );
System.out.println("contentX: "+contentX+"  contentY: "+contentY);
System.out.println("contentWidth: "+contentWidth+"  contentHeight: "+contentHeight);
System.out.println("gridRowCount: "+gridRowCount);
System.out.println("gridColCount: "+gridColCount);
System.out.println("gridPageCount: "+gridPageCount);
System.out.println("topRowIndex: "+topRowIndex);
System.out.println("curPageIndex: "+curPageIndex);
System.out.println("curPageSize: "+curPageSize);
}
//******************************************************************************

		//format the raw data
		formatMultipagedData();	//only multipaged format suppotred at this time

		//repaint this component
		this.repaint();
	}

	void formatMultipagedData()
	{
		int i, j, textM=3, textHeight=gRowHeight-5, cX=contentX, cY=contentY, pageRow=1;

		for(i=0; i < gridColCount; i++)
		{
		//set Text Locations
		for(j=0; j < gridRowCount; j++)
		{
			colDataX[i][j] = cX + gColBounds[i].x+textM;
			colDataY[i][j] = cY + gColBounds[i].y + textHeight;
			cY += gRowHeight;  pageRow++;
			if(pageRow > xPageSize) { pageRow=1; cY=contentY; }
//System.out.println("colDataX["+i+"]["+j+"] : "+colDataX[i][j]+"   colDataY["+i+"]["+j+"] : "+colDataY[i][j]);
		}
		//set position vars for next column
		cX += gColWidth[i];  cY=contentY;  pageRow=1;
		}
	}


	public void setText(String[][] gridData)
	{
		int i, j;

		this.reset();
		gridRowCount = gridData.length;
		gridColData = new String[gridColCount][gridRowCount];
		colDataX = new int[gridColCount][gridRowCount];
		colDataY = new int[gridColCount][gridRowCount];

		//populate the data from array
		for(i=0; i < gridRowCount; i++)
			for(j=0; j < gridColCount; j++)
				gridColData[j][i] = gridData[i][j];

		updateView();
	}

	public String[][] getText()	//return Col-Row Data
	{
		return gridColData;
	}

	public String[] getText(int row)
	{
		String[] rData = new String[gridColCount];
		for(int i=0; i < gridColCount; i++) rData[i] = gridColData[i][row];
		return rData;
	}

	public String getText(int row, int col)
	{
		return gridColData[col][row];
	}

	public int getRowCount() { return gridRowCount; }

	public int findText(String text, int col)
	{
		int fRow = -1;
		for(int i=0; i < gridRowCount; i++)
			if((gridColData[col][i]).equalsIgnoreCase(text)) { fRow = i; break; }
		if(fRow < 0) return fRow;

		//move the page to one having this value
		int tIndex = fRow/xPageSize;
		if(tIndex != curPageIndex)
		{	curPageIndex = tIndex;
			topRowIndex = tIndex*xPageSize;
			if(tIndex > 0) btnPrevPage.setEnabled(true);
			if(tIndex < gridPageCount-1) btnNextPage.setEnabled(true);
			if(curPageIndex == gridPageCount-1) curPageSize = gridRowCount - topRowIndex;
			this.repaint();
		}
		return fRow;
	}


	//Event Handler for All Actions
	public void actionPerformed(ActionEvent e)
	{
		Object eventObj = e.getSource();

		if(eventObj == btnNextPage)
		{	curPageIndex++;
			topRowIndex += xPageSize;
			btnPrevPage.setEnabled(true);
			if(curPageIndex == gridPageCount-1)
			{	btnNextPage.setEnabled(false);
				curPageSize = gridRowCount - topRowIndex;
			}
			this.repaint();
		}
		else
		{	curPageIndex--;
			topRowIndex -= xPageSize;
			curPageSize  = xPageSize;
			btnNextPage.setEnabled(true);
			if(curPageIndex < 1) btnPrevPage.setEnabled(false);
			this.repaint();
		}
	}

	//------ Implementation of MouseListener Interface------
	public void mouseClicked(MouseEvent e)
	{
		//System.out.println("MouseClicked");
	}

	public void mouseEntered(MouseEvent e)
	{
		//System.out.println("mouseEntered");
		//e.getComponent().setForeground(hrefColorH);
	}

	public void mouseExited(MouseEvent e)
	{
		//System.out.println("mouseExited");
		mouseInColN = -1;  mouseInRowN = -1; //new mouse column
		if(mouseInCol != mouseInColN || mouseInRow != mouseInRowN)
		{ isLinkColEvent = true;  this.repaint(); }
	}

	public void mousePressed(MouseEvent e)
	{
		if(linkListener == null) return;
		if(mouseInCol != -1)
		{	int cmdID = mouseInCol + ((mouseInRow+topRowIndex)*100);
			ActionEvent a = new ActionEvent(this, cmdID, linkActionCommand);
			linkListener.actionPerformed(a);
			//System.out.println("First: "+ActionEvent.ACTION_FIRST+"  Last: "+ActionEvent.ACTION_LAST+"  ActionPerformed: "+ActionEvent.ACTION_PERFORMED);
			//System.out.println("EventObj: "+a.getSource()+"  EventID"+a.getID()+"  Command: "+a.getActionCommand());
		}
	}

	public void mouseReleased(MouseEvent e)
	{

	}

	//Implementation of MouseMotionListener Interface
	public void mouseDragged(MouseEvent e) { }

	public void mouseMoved(MouseEvent e)
	{
		int i, j;
		Rectangle cR = new Rectangle(contentX, contentY, gColWidth[0], gRowHeight);
		Point mPt = e.getPoint();

		mouseInColN = -1;  mouseInRowN = -1; //new mouse column
		for(i=0; i < gridColCount; i++)  //check if any Link Column has mouse in it
		{	cR.width = gColWidth[i];
			if(isColDataLink[i])
			for(j=0; j < curPageSize; j++)
			{	if(cR.contains(mPt)) { mouseInColN=i; mouseInRowN=j; break; }
				cR.y += gRowHeight;
			}
			cR.x += gColWidth[i];  cR.y = contentY;
		}

		if(mouseInCol != mouseInColN || mouseInRow != mouseInRowN)
		{ isLinkColEvent = true;  this.repaint(); }
	}

}
