import java.awt.Component;

public class XLayout
{
	//Layout Constants
	public static final int ROWDEF  = 8100;
	public static final int COLDEF  = 8101;
	public static final int LDEF    = 8102;
	public static final int INDENT  = 8103;
	public static final int VGAP    = 8104;
	public static final int HGAP    = 8105;
	public static final int SKIPROW = 8106;
	public static final int SKIPCOL = 8107;
	public static final int RESET   = 8108;

	//Layout specific
	private int cRowHeight  = 20;
	private int cRowHeightM = 20;
	private int cRowMLeft   = 0;
	private int cRowMTop    = 0;
	private int cRowMRight  = 0;
	private int cRowMBot    = 0;
	private int cColCount   = 0;
	private int[] cColWidth;
	private int[] cColWidthM;
	private int[] cColLabelD;
	private int cIndent = 0;

	//Cursor specific
	private int cursorPos = 0;
	private int cursorX   = 0;
	private int cursorY   = 0;

	java.awt.Rectangle UIRect = new java.awt.Rectangle();
	java.awt.Container formContainer;

	public XLayout() { }

	public XLayout(java.awt.Container c) { formContainer = c; }

	//set the container for this layout
	public void setContainer(java.awt.Container c) { formContainer = c; }

	//set row properties sequenced as: RowHeight, spLeft, spTop, spRight, spBot
	public void setRowProperties(int pCount, int[] Prop)
	{
		if(pCount > 0) cRowHeight = Prop[0];
		if(pCount > 1) cRowMLeft  = Prop[1];
		if(pCount > 2) cRowMTop   = Prop[2];
		if(pCount > 3) cRowMRight = Prop[3];
		if(pCount > 4) cRowMBot   = Prop[4];
		cRowHeightM = cRowHeight-cRowMTop-cRowMBot;
	}

	//set column widths
	public void setColProperties(int cCount, int[] cWidth)
	{
		if(cCount < 1) return;
		cColCount   = cCount;
		cColWidth   = new int[cCount];
		cColWidthM  = new int[cCount];
		cColLabelD  = new int[cCount];
		for(int i=0; i < cCount; i++)
		{	cColWidth[i]  = cWidth[i];
			cColWidthM[i] = cWidth[i]-cRowMLeft-cRowMRight;
		}
		if(cursorPos > 0)
		{	cursorPos=0; cursorX=cIndent;
			cursorY += cRowHeight;
		}
	}

	//set current Indent
	public void setIndent(int ind)
	{
		cIndent = ind;
		if(cursorPos == 0) cursorX = cIndent;
	}

	public void skipCol(int n)
	{
		for(int i=0; i < n; i++)
		{
		cursorX += cColWidth[cursorPos];
		cursorPos++;
		if(cursorPos >= cColCount)
		{	cursorPos=0; cursorX=cIndent;
			cursorY += cRowHeight;
		}
		}
	}

	public void skipRow(int n)
	{
		cursorPos=0; cursorX=cIndent;
		cursorY += n*cRowHeight;
	}

	public void reset()
	{
		if(formContainer != null) formContainer.removeAll();

		//Reset Layout
		cRowHeight  = 20;
		cRowHeightM = 20;
		cRowMLeft   = 0;
		cRowMTop    = 0;
		cRowMRight  = 0;
		cRowMBot    = 0;
		cColCount   = 0;
		cIndent     = 0;

		cursorPos = 0;
		cursorX   = 0;
		cursorY   = 0;
	}

	public java.awt.Rectangle getXBounds()
	{
		if(cColCount < 1) return null;

		int i = cursorPos;

		UIRect.x = cursorX+cRowMLeft;
		UIRect.y = cursorY+cRowMTop;
		UIRect.width  = cColWidthM[i];
		UIRect.height = cRowHeightM;

		cursorX += cColWidth[i];
		cursorPos++;
		if(cursorPos >= cColCount)
		{	cursorPos=0; cursorX=cIndent;
			cursorY += cRowHeight;
		}

		return UIRect;
	}

	public boolean handleXItem(int[] itemData)
	{
		int itemID  = itemData[0];
		int itemLen = itemData.length-1;
		int[] xData;

		switch(itemID)
		{
			case XLayout.ROWDEF:
				xData = new int[itemLen];
				System.arraycopy(itemData, 1, xData, 0, itemLen);
				setRowProperties(itemLen, xData);
				return true;

			case XLayout.COLDEF:
				xData = new int[itemLen];
				System.arraycopy(itemData, 1, xData, 0, itemLen);
				setColProperties(itemLen, xData);
				return true;

			case XLayout.LDEF:
				for(int i=0; i < itemLen; i++) cColLabelD[i] = itemData[1+i];
				return true;


			case XLayout.INDENT:
				setIndent(itemData[1]);
				return true;

			case XLayout.HGAP:
				cursorX += itemData[1];
				return true;

			case XLayout.VGAP:
				cursorY += itemData[1];
				return true;

			case XLayout.SKIPROW:
				if(itemLen < 1) skipRow(1);
				else skipRow(itemData[1]);
				return true;

			case XLayout.SKIPCOL:
				if(itemLen < 1) skipCol(1);
				else skipCol(itemData[1]);
				return true;

			case XLayout.RESET:
				reset();
				return true;
		}

		return false;
	}

	void addComponent(Component c)
	{
		if(formContainer == null) return;
		java.awt.Rectangle R = getXBounds();
		c.setBounds(R);
		formContainer.add(c);
	}

	void addLabeledComponent(Component Lb, Component c)
	{
		if(formContainer == null) return;

		int labelD = cColLabelD[cursorPos];	//must get it before calling getUIRect()
		java.awt.Rectangle R = getXBounds();
		int tw = R.width;
		R.width = labelD;
		Lb.setBounds(R);
		R.x += labelD; R.width = tw-labelD;
		c.setBounds(R);
		formContainer.add(Lb);
		formContainer.add(c);
	}
}
