import java.awt.*;

public class XToolkit
{
	static final int COLORSCHEME_NORMAL3D   = 1;
	static final int COLORSCHEME_SHARP3D    = 2;
	static final int COLORSCHEME_SOFT3D     = 4;
	static final int COLORSCHEME_DIALOGEDGE = 8;

	static final int MESSAGE_PROGRESS = 1;
	static final int MESSAGE_STATIC   = 2;
	static final int MESSAGE_STATIC2  = 3;

	static Color brightEdgeColor = Color.white;
	static Color darkEdgeColor   = Color.gray;
	static Color shadowColor     = Color.black;
	static XMsg wMsg=null;
	static Dimension wMsgD;
	static String wMsgText="", wMsgText2="";
	static int wMsgTextWidth=0, wMsgTextWidth2=0;
	static ProgressListener pListener=null;
	static int msgType = MESSAGE_PROGRESS;

	static final Font TitleBarFont = new Font("SansSerif", Font.BOLD, 12);
	static final Font msgFont = new Font("SansSerif", Font.BOLD, 12);
	static final FontMetrics msgFM = Toolkit.getDefaultToolkit().getFontMetrics(msgFont);


	static int[] clockX1 = { 23, 20, 12, 0, -12, -20 };
	static int[] clockY1 = { 0, 12, 20, 23, 20, 12 };
	static int[] clockX2 = { 30, 26, 15, 0, -15, -26 };
	static int[] clockY2 = { 0, 15, 26, 30, 26, 15 };
	static int prevX, prevY;

	public static void setBrightEdgeColor(Color c)
	{
		brightEdgeColor = c;
	}

	public static void setDarkEdgeColor(Color c)
	{
		darkEdgeColor = c;
	}

	public static void setShadowColor(Color c)
	{
		shadowColor = c;
	}

	public static void setColorScheme(int scheme)
	{
		switch(scheme)
		{
			case COLORSCHEME_NORMAL3D:
					brightEdgeColor = Color.white;
					darkEdgeColor   = Color.gray;
					return;

			case COLORSCHEME_SHARP3D:
					brightEdgeColor = Color.white;
					darkEdgeColor   = Color.black;
					return;

			case COLORSCHEME_SOFT3D:
					brightEdgeColor = Color.lightGray;
					darkEdgeColor   = Color.gray;
					return;

			case COLORSCHEME_DIALOGEDGE:
					brightEdgeColor = Color.lightGray;
					darkEdgeColor   = Color.black;
					return;
		}
	}

	public static void draw3DBox(Graphics g, int x, int y, int width, int height, boolean r)
	{
		if(r) g.setColor(brightEdgeColor);
		else  g.setColor(darkEdgeColor);
		g.drawLine(x, y, x+width-1, y);
		g.drawLine(x, y, x, y+height-1);
		if(r) g.setColor(darkEdgeColor);
		else  g.setColor(brightEdgeColor);
		g.drawLine(x, y+height-1, x+width-1, y+height-1);
		g.drawLine(x+width-1, y, x+width-1, y+height-1);
	}

	public static void draw3DBorder(Graphics g, int x, int y, int width, int height, int t, boolean r)
	{
		draw3DBox(g, x, y, width+2*t, height+2*t, r);
		draw3DBox(g, x+t, y+t, width, height, !r);
	}

	public static void draw3DLineH(Graphics g, int x, int y, int len, int thickness, boolean r)
	{
		if(r) g.setColor(brightEdgeColor);
		else  g.setColor(darkEdgeColor);
		g.drawLine(x, y, x+len-1, y);
		if(r) g.setColor(darkEdgeColor);
		else  g.setColor(brightEdgeColor);
		g.drawLine(x, y+thickness, x+len-1, y+thickness);
	}

	public static void draw3DLineV(Graphics g, int x, int y, int len, int thickness, boolean r)
	{
		if(r) g.setColor(brightEdgeColor);
		else  g.setColor(darkEdgeColor);
		g.drawLine(x, y, x, y+len-1);
		if(r) g.setColor(darkEdgeColor);
		else  g.setColor(brightEdgeColor);
		g.drawLine(x+thickness, y, x+thickness, y+len-1);
	}


	public static void initMessaging(Container c)
	{
		wMsg = new XMsg();
		wMsg.setBounds(310, 90, 340, 170);
		wMsg.setBackground(Color.lightGray);
		wMsg.setName("Billing Portal");
		c.add(wMsg);
		wMsgD = wMsg.getSize();
	}

	public static void showProgressMessage(String msg)
	{
		msgType = MESSAGE_PROGRESS;
		if(msg == null) wMsgTextWidth = 0;
		else { wMsgText = msg; wMsgTextWidth = msgFM.stringWidth(msg); }
		wMsg.setVisible(true);
		wMsg.startTimer();
	}

	public static void hideProgressMessage()
	{
		wMsgText = "";
		wMsg.setVisible(false);
		wMsg.stopTimer();
	}

	public static void setProgressListener(ProgressListener pL) { pListener = pL; }

	public static void showMessage(String msg)
	{
		msgType = MESSAGE_STATIC;
		if(msg == null) wMsgTextWidth = 0;
		else { wMsgText = msg; wMsgTextWidth = msgFM.stringWidth(msg); }
		wMsg.staticMessage();
		wMsg.setVisible(true);
	}

	public static void showMessage(String msg1, String msg2)
	{
		msgType = MESSAGE_STATIC2;
		if(msg1 == null) wMsgTextWidth = 0;
		else { wMsgText = msg1; wMsgTextWidth = msgFM.stringWidth(msg1); }
		if(msg2 == null) wMsgTextWidth2 = 0;
		else { wMsgText2 = msg2; wMsgTextWidth2 = msgFM.stringWidth(msg2); }
		wMsg.staticMessage();
		wMsg.setVisible(true);
	}

	public static void closeMessage()
	{
		wMsgText = "";
		wMsg.setVisible(false);
		if(msgType == MESSAGE_PROGRESS)
		{	wMsg.stopTimer();
			if(pListener != null) pListener.cancelProgress();
		}
	}

	public static void drawMsgbox(Graphics g)
	{
		setColorScheme(COLORSCHEME_DIALOGEDGE);
		draw3DBox(g, 0, 0, wMsgD.width, wMsgD.height, true);
		setColorScheme(COLORSCHEME_NORMAL3D);
		draw3DBox(g, 1, 1, wMsgD.width-2, wMsgD.height-2, true);
		g.setColor(Color.blue);
		g.fillRect(3, 3, wMsgD.width-6, 20);
		g.setColor(Color.white);
		g.setFont(TitleBarFont);
		g.drawString(wMsg.getName(), 8, 17);
		g.setColor(Color.black);
		g.setFont(msgFont);
		if(msgType == MESSAGE_PROGRESS)
			g.drawString(wMsgText, (wMsgD.width-wMsgTextWidth)/2, 120);
		else if(msgType == MESSAGE_STATIC)
		{	g.drawString(wMsgText, (wMsgD.width-wMsgTextWidth)/2, 100);
			return;
		}
		else
		{	g.drawString(wMsgText, (wMsgD.width-wMsgTextWidth)/2, 80);
			g.drawString(wMsgText2, (wMsgD.width-wMsgTextWidth2)/2, 100);
			return;
		}

		//draw the clock
		g.setColor(Color.black);
		g.drawOval(wMsgD.width/2-30, 30, 60, 60);
		int cx = wMsgD.width/2, cy=60;
		for(int i=0; i < 6; i++)
		{	g.drawLine(cx+clockX1[i], cy+clockY1[i], cx+clockX2[i], cy+clockY2[i]);
			g.drawLine(cx-clockX1[i], cy-clockY1[i], cx-clockX2[i], cy-clockY2[i]);
		}
		prevX=0;  prevY=0;
		g.setXORMode(Color.blue);
		g.setColor(Color.green);
		g.drawLine(cx, cy, cx+prevX, cy+prevY);
		g.setPaintMode();
		/*g.setColor(Color.red);
		g.drawLine(cx, cy, cx+20, cy);
		g.drawLine(cx, cy, cx+17, cy+10);
		g.drawLine(cx, cy, cx+10, cy+17);
		g.drawLine(cx, cy, cx, cy+20);*/
	}

	public static void updateMsgClock(Graphics g, int tick)
	{
		g.setXORMode(Color.blue);
		g.setColor(Color.green);
		int x = (int)(Math.cos(Math.PI*tick/6)*20);
		int y = (int)(Math.sin(Math.PI*tick/6)*20);
		int cx = wMsgD.width/2,  cy=60;
		g.drawLine(cx, cy, cx+prevX, cy+prevY);
		g.drawLine(cx, cy, cx+x, cy+y);
		prevX = x;  prevY = y;
		g.setPaintMode();
	}

}
