/*
 * @(#)StreamEncryptor.java	1.0
 * Copyright (c) iBasis Inc.
 *
 */
//package portal.servletbeans;

import java.io.*;
import java.net.*;

/**
 * This is a simple Stream Encryptor which uses a combination of
 * random and mathematical functions to generate a sequence of
 * bytes (usually 64) which are added to the actual data to
 * achieve pseudorandom encryption
 */
public class EncryptedOutputStream extends OutputStream
{

	private java.util.Random rndGenerator = new java.util.Random();
	private static final int PACKET_LENGTH=64;
	private byte[] RandomPacket     = new byte[PACKET_LENGTH];
	private byte[] PacketScrambler  = new byte[PACKET_LENGTH];
	private byte[] PacketScrambler2 = new byte[PACKET_LENGTH];
	private int xStartPos = 23;
	private int xEndPos   = 41;

	private OutputStream destStream = null;
	private boolean isUninitialized = true;
	private int cursorPos = 0;


	public EncryptedOutputStream()
	{
	}

	public void setOutputStream(OutputStream os)
	{
		destStream = os;
		isUninitialized = true;
	}


	private void resetEncryption()
	{
		rndGenerator.setSeed(rndGenerator.nextLong());
		rndGenerator.nextBytes(RandomPacket);
		xStartPos = RandomPacket[23]/4;	//in the range of -32 to 32
		if(xStartPos < 0) xStartPos = -xStartPos;
		xEndPos = RandomPacket[PACKET_LENGTH-xStartPos-1]/4;	// range of -32 to 32
		if(xEndPos < 0) xEndPos = -xEndPos;
		rndGenerator.setSeed(rndGenerator.nextLong());
		rndGenerator.nextBytes(RandomPacket);
		rndGenerator.setSeed(rndGenerator.nextLong());
		rndGenerator.nextBytes(PacketScrambler);
		rndGenerator.nextBytes(PacketScrambler2);
	}

	private void initStream() throws IOException
	{
		//generate new encryption sequence
		resetEncryption();

		//write starting position byte
		destStream.write(xStartPos);
		//write start scrambler bytes upto xStartPos
		destStream.write(PacketScrambler, 0, xStartPos);
		//write ending position byte
		destStream.write(xEndPos);
		//write end scrambler bytes upto xEndPos
		destStream.write(PacketScrambler2, 0, xEndPos);
		//write Encryption Key
		destStream.write(RandomPacket, 0, PACKET_LENGTH);

		//set flag to indicate that stream has been initialized
		isUninitialized = false;
		cursorPos = 0;

//System.out.println("\nEncrypted Output: "+xStartPos+"  "+xEndPos);
//System.out.println("Random Sequence: ");
//for(int i=0; i < 64; i++) System.out.print((int)(RandomPacket[i])+"  ");
//System.out.println("");
	}


	//Implementation of OutputStream
	public void write(int b) throws IOException
	{
		if(isUninitialized) initStream();
		if(cursorPos < PACKET_LENGTH)
		{
			destStream.write((int)(b+RandomPacket[cursorPos]));
			cursorPos++;
		}
		else
		{
			destStream.write((int)(b+RandomPacket[0]));
			cursorPos = 1;
		}
	}

	public void write(byte b[]) throws IOException
	{
		if(isUninitialized) initStream();
		int tLen = b.length;

//System.out.print("Original Output Data: "+tLen+" :\n");
//for(int i=0; i < tLen; i++) System.out.print((int)(b[i])+"  ");
//System.out.println("");

		for(int i=0; i < tLen; i++)
		{
			if(cursorPos < PACKET_LENGTH)
			{
			  b[i] += RandomPacket[cursorPos];
			  cursorPos++;
			}
			else
			{
			  b[i] += RandomPacket[0];
			  cursorPos = 1;
			}
		}

//System.out.print("Encrypted Output Data: "+tLen+" :\n");
//for(int i=0; i < tLen; i++) System.out.print((int)(b[i])+"  ");
//System.out.println("");

		destStream.write(b);
	}

	public void write(byte b[], int off, int len) throws IOException
	{
		if(isUninitialized) initStream();
		for(int i=0; i < len; i++)
		{
			if(cursorPos < PACKET_LENGTH)
			{
			  b[off+i] += RandomPacket[cursorPos];
			  cursorPos++;
			}
			else
			{
			  b[off+i] += RandomPacket[0];
			  cursorPos = 1;
			}
		}
		destStream.write(b, off, len);
	}

	public void close() throws IOException
	{
		isUninitialized = true;
		destStream.close();
	}


	//For Testing purposes only
	/*public static void main(String[] args)
	{
		EncryptedOutputStream te = new EncryptedOutputStream();
		try {
		Socket ts = new Socket("localhost", 8081);

		String data = "My Name is Ash Wini...";

		long st = System.currentTimeMillis();

		te.setOutputStream(ts.getOutputStream());
		te.write(data.getBytes());
		te.close();
		ts.close();
		} catch (Exception e) { System.out.println("Error: "+e.toString());  }
	}
	*/
}
