import java.io.*;
import java.net.*;


public class HttpCommunicator implements Runnable, ProgressListener
{
	private URL cURL;
	private URLConnection cURLConnection;
	private boolean doEncryption=true;
	private boolean reqCancelled=false;

	private XMLParser cParser = new XMLParser();
	private Document  inDoc, outDoc;

	private HttpListener cListener;
	private int cDataID = -1;

	private EncryptedInputStream  eIn  = new EncryptedInputStream();
	private EncryptedOutputStream eOut = new EncryptedOutputStream();


	public HttpCommunicator() { }

	public HttpCommunicator(String url)
	{
		setURL(url);
	}

	public void setURL(String url)
	{
		try
		{
			cURL = new URL(url);
		} catch (Exception e)
		{ System.out.println("Error: \n  HttpCommunicator error: "+e.toString()); }
	}

	public void setEncryption(boolean en) { doEncryption = en; }


	public String processRequest(String reqStr)
	{
		if(cURL == null) return null;
		int blkSize=8192;
		byte[] xBytes = new byte[blkSize];
		StringBuffer xBuf = new StringBuffer();

		reqCancelled=false;
		try
		{	cURLConnection = cURL.openConnection();
			cURLConnection.setRequestProperty("Content-Type", "application/x-billing");
			if(reqStr != null)
			{	cURLConnection.setDoOutput(true);
				OutputStream out = cURLConnection.getOutputStream();
				if(doEncryption) { eOut.setOutputStream(out); out=eOut; }
				out.write(reqStr.getBytes());
				out.close();
			}
			InputStream in = (InputStream)(cURLConnection.getContent());
			if(doEncryption) { eIn.setInputStream(in); in=eIn; }
			while(true)
			{	int nb = in.read(xBytes, 0, blkSize);
				if(nb < 0) break;
				if(nb > 0) xBuf.append(new String(xBytes, 0, nb));
			}
			in.close();
			return xBuf.toString();
		}  catch (Exception e)
		{  System.out.println("Error: \n  HttpCommunicator error: "+e.toString()); }

		return null;
	}

	public void processXMLRequest(Document doc, HttpListener cL, int dataID)
	{
		reqCancelled=false;
		Thread xThread = new Thread(this);
		inDoc   = doc;
		outDoc  = null;
		cListener = cL;
		cDataID = dataID;
		xThread.start();
	}

	public void cancelProgress() { reqCancelled=true; }

	public void loadXMLDocument()
	{
		if(cURL == null) return;

		try
		{	cURLConnection = cURL.openConnection();
			cURLConnection.setDoOutput(true);
			cURLConnection.setRequestProperty("Content-Type", "application/x-billing");
			OutputStream out = cURLConnection.getOutputStream();
			if(doEncryption) { eOut.setOutputStream(out); out=eOut; }
			cParser.outputXML(inDoc, out);
			out.close();
			InputStream in = (InputStream)(cURLConnection.getContent());
			if(doEncryption) { eIn.setInputStream(in); in=eIn; }
			if(!reqCancelled) outDoc = cParser.parseStream(in);
			in.close();
		}  catch (Exception e)
		{  System.out.println("Error: \n  HttpCommunicator error: "+e.toString()); }
	}

	public void run()
	{
		loadXMLDocument();
		if(!reqCancelled && cListener != null) cListener.receiveHttpData(outDoc, cDataID);
		cListener = null;
	}

}
